rm(list=ls(all=TRUE))
library(zoo)
library(xtable)
library(e1071)
source("utilities.r")


#======================================================================
# DEFINE THE VARIABLES 

# logical variable (TRUE or FALSE), defines whether to simulate with look-ahead bais or not
look.ahead.bias <- TRUE

# define the length of the averaging period, the number of months
window.length <- 24

# define the level of transaction costs
tc <- 0.005 

# select the dataset
dataSet <- "1"

# define the historical period
start <- as.yearmon(paste(1960,1), "%Y %m")
end <- as.yearmon(paste(2011,12), "%Y %m")

switch(dataSet,
       "1" = {
         data <- read.table("Portfolios_Formed_on_ME.txt", header=TRUE)
         dataset.name <- "Size-sorted portfolios"
       },
       
       "2" = {
         data <- read.table("Portfolios_Formed_on_BE-ME.txt", header=TRUE)
         dataset.name <- "Book-to-market-sorted portfolios"
       },
       
       "3" = {
         data <- read.table("10_Portfolios_Prior_12_2.txt", header=TRUE)
         dataset.name <- "Momentum-sorted portfolios"
       },
       {
         stop("Unknown data set!")
       }       
)

info.str<-paste("MA(",window.length,") strategy",sep="")
if(look.ahead.bias==TRUE) {
  info.str<-paste(info.str, "With look-ahead bias")
} else  {
  info.str<-paste(info.str, "WithOut look-ahead bias")
}


#======================================================================
# READ THE DATA

# read the portfolio returns
dates <- str2date(as.character(data$Date))
nCol <- ncol(data)
ret <- data[,(nCol-9):nCol]/100

# read the factor returns
data <- read.table("F-F_Research_Data_Factors.txt", header=TRUE)
rf <- data$RF/100
mkt <- data$MKTMRF/100 + rf
smb <- data$SMB/100
hml <- data$HML/100
data <- read.table("F-F_Momentum_Factor.txt", header=TRUE)
mom <- data$MOM/100

# select the data in the required historical period
ret <- as.matrix(ret[dates >= start & dates <= end,])
mkt <- mkt[dates >= start & dates <= end]
rf  <- rf[dates >= start & dates <= end]
smb <- smb[dates >= start & dates <= end]
hml <- hml[dates >= start & dates <= end]
mom <- mom[dates >= start & dates <= end]

# construct the FFC 4-factor model returns
FFC4 <- cbind(mkt-rf,smb,hml,mom)
# delete the returns in the lookback period
FFC4 <- FFC4[(window.length+1):length(rf),]

#======================================================================
# SIMULATE THE RETURNS TO THE MA(L) STRATEGY 

bh.ret <- vector()
ma.ret <- vector()

for(i in 1:10) {
  r <- ret[,i]
  if(look.ahead.bias==TRUE) {
    res <- sim.wrong(r, rf, lookback=window.length, tc=tc)
  } else {
    if(look.ahead.bias==FALSE) {
      res <- sim.correct(r, rf, lookback=window.length, tc=tc)
    } else stop("look.ahead.bias can be either TRUE or FALSE!")
  }
  bh.ret <- cbind(bh.ret, res$mkt)
  ma.ret <- cbind(ma.ret, res$ret)
}
rf  <- res$rf 

#======================================================================
# COMPUTE THE PERFORMANCES

col.names <- c("Mean.BH", "Std.BH", "Skew.BH", "Sharpe.BH", "Alpha", "pval",
           "Mean.MA", "Std.MA", "Skew.MA", "Sharpe.MA", "pval", "Alpha.MA", "pval")
n <- length(col.names)
results <- matrix(nrow=10,ncol=n)

for(i in 1:10) {
  # select the BH and MA returns
  rp <- bh.ret[,i]
  ra <- ma.ret[,i]
  
  # evaluate the performance of BH strategy
  results[i,1] <- mean(rp)*100*12
  results[i,2] <- sd(rp)*100*sqrt(12)
  results[i,3] <- skewness(rp)
  
  results[i,4] <- Sharpe(rp-rf)
  
  res <- ols(rp-rf,FFC4)
  results[i,5] <- res$beta[1]*100*12
  results[i,6] <- res$pval[1]
  
  # evaluate the performance of MA strategy
  results[i,7] <- mean(ra)*100*12
  results[i,8] <- sd(ra)*100*sqrt(12)
  results[i,9] <- skewness(ra)
  
  results[i,10] <- Sharpe(ra-rf)
  results[i,11] <- SharpeTest(rp-rf,ra-rf)
  
  res <- ols(ra-rf,FFC4)
  results[i,12] <- res$beta[1]*100*12
  results[i,13] <- res$pval[1]
}


# print the results as a table
row.names <- c("Low", "2", "3", "4", "5", "6", "7", "8", "9", "High")

res.df <- as.data.frame(results)
colnames(res.df) <- col.names
rownames(res.df) <- row.names

xtab<-xtable(res.df)
print(xtab, include.rownames=TRUE)

# make box plots
leg.names <- c("BH","MA")
colours <- c("red", "green")

SR <- t(results[,c(4,10)])
barplot(SR, beside=TRUE, col=colours, names.arg=row.names,
        ylab="Sharpe ratio", xlab=dataset.name, main=info.str)
legend("topright", leg.names, fill=colours, bg="white")

Alpha <- t(results[,c(5,12)])
barplot(Alpha, beside=TRUE, col=colours, names.arg=row.names,
        ylab="Alpha FFC4, %", xlab=dataset.name, main=info.str)
legend("topright", leg.names, fill=colours, bg="white")

