RollSma <- function(price, lookback) {
  # computes the trading signal based on
  # the rolling Simple Moving Average
  nobs <- length(price)
  sma <- rep(0,nobs)
  for (i in lookback:nobs) sma[i] <- price[i] - mean(price[(i-lookback+1):i])
  return(sma)
}

sim.correct <- function(mkt, rf, lookback=10, tc=0) {
  # this function simulates the returns to the active trading strategy
  # WITHOUT the look-ahead bias
  # Inputs:
  # mkt   - market returns (or returns to the buy-and-hold strategy)
  # rf    - risk-free rate of return
  # lookback - the size of the averaging window
  # tc    - amount of transaction costs
  
  nobs <- length(mkt)
  if (length(rf) != nobs) stop("Different numbers of market and risk-free returns!")
  
  price <- cumprod(1+mkt) # compute the price using returns
  signal <- RollSma(price, lookback) # compute the trading signal
  
  ret <- rep(0, nobs) # to hold active returns
  for (i in lookback:(nobs-1)) {
    # simulate returns to the active strategy
    if (i == lookback) {
      if (signal[i]>0) {
        # first trading without transaction costs
        ret[i+1] <- mkt[i+1] # hold in stocks
      } else ret[i+1] <- rf[i+1] # hold in cash
    } else {
      if (signal[i]>0) {
        # hold in stocks
        if (signal[i-1]>0) ret[i+1]<-mkt[i+1] else ret[i+1]<-mkt[i+1]-tc
      } else {
        # hold in cash
        if (signal[i-1]>0) ret[i+1]<-rf[i+1]-tc else ret[i+1]<-rf[i+1]
      }
    }
  }  
  
  # now delete the observations in the first lookback period
  ret <- ret[-(1:(lookback))]
  mkt <- mkt[-(1:(lookback))]
  rf <- rf[-(1:(lookback))]
  
  # return a list with results
  result <- list(ret=ret, mkt=mkt, rf=rf)
  return(result)
}

sim.wrong <- function(mkt, rf, lookback=10, tc=0) {
  # this function simulates the returns to the active trading strategy
  # WITH the look-ahead bias
  # Inputs:
  # mkt   - market returns (or returns to the buy-and-hold strategy)
  # rf    - risk-free rate of return
  # lookback - the size of the averaging window
  # tc    - amount of transaction costs
  
  nobs <- length(mkt)
  if (length(rf) != nobs) stop("Different numbers of market and risk-free returns!")
  
  price <- cumprod(1+mkt) # compute the price using returns
  signal <- RollSma(price, lookback) # compute the trading signal
  
  ret <- rep(0, nobs) # to hold active returns
  for (i in lookback:nobs) {
    # simulate returns to the active strategy
    if (i == lookback) {
      if (signal[i]>0) {
        # first trading without transaction costs
        ret[i] <- mkt[i] # hold in stocks
      } else ret[i] <- rf[i] # hold in cash
    } else {
      if (signal[i]>0) {
        # hold in stocks
        if (signal[i-1]>0) ret[i]<-mkt[i] else ret[i]<-mkt[i]-tc
      } else {
        # hold in cash
        if (signal[i-1]>0) ret[i]<-rf[i]-tc else ret[i]<-rf[i]
      }
    }
  }  

  # now delete the observations in the first lookback period
  ret <- ret[-(1:(lookback))]
  mkt <- mkt[-(1:(lookback))]
  rf <- rf[-(1:(lookback))]
  
  # return a list with results
  result <- list(ret=ret, mkt=mkt, rf=rf)
  return(result)
}

Sharpe <- function(ex) {
  # computes and returns the annualized Sharpe ratio
  # input <- ex - excess returns of a portfolio
  return(mean(ex)/sd(ex)*sqrt(12))
}

SharpeTest <- function(ex1, ex2) {
  # test for equality of two Sharpe ratios
  # ex1 - excess returns of the first portfolio
  # ex2 - excess returns of the second portfolio
  # returns the p-value of the test

  if (length(ex1) != length(ex2))
    stop("Different lengths of two return series!")
  
  SR1 <- mean(ex1)/sd(ex1)
  SR2 <- mean(ex2)/sd(ex2)
  ro <- cor(ex1,ex2)
  n <- length(ex1)
  z <- (SR2-SR1)/sqrt( (2*(1-ro)+0.5*(SR1^2+SR2^2-2*SR1*SR2*ro^2))/n )
  pval <- 2*pnorm(-abs(z))
  return(pval)
}

ols <- function(Y, X) {
  # this function regresses measurements in the vector Y
  # on values in the matrix X using a multiple OLS model
  #
  # inputs:
  # Y    			      N x 1 vector of responses
  # X  			        N x (k-1) matrix of observations of (k-1) independent
  #                 variables, it does not include the column of ones
  #
  # output is a list containing the following regression statistics
  # beta            k x 1 vector of regression coefficients
  # pval            k x 1 vector of p-values of beta

  X <- cbind(1,as.matrix(X)) # add vector of ones as the first column
  Y <- as.vector(Y) # just for the sake of robustness
  n <- length(Y)  # number of observations
  k <- ncol(X)    # number of independent variables
  
  xxi <- solve(t(X) %*% X)
  xy <- t(X) %*% Y
  beta <- as.vector(xxi %*% xy)         # regression coefficients
  yhat <- as.vector(X %*% beta)         # fitted values of Y
  resid <- Y - yhat                     # model residuals
  rss <-  as.numeric(resid %*% resid)   # residual sum of squares 
  se <-sqrt(diag(rss/(n-k)*xxi))        # standard errors of estimation of beta
  tstat <- as.vector(beta/se)           # t-statistics of beta
  pval <- 2*(1-pt(abs(tstat),n-k))      # p-values of beta

  # construct the list to return the results
  stats <- list(beta=beta, yhat=yhat, resid=resid, se=se, tstat=tstat, pval=pval)
  return(stats)                 
  
}

str2date <- function(date) {
  # convert string date to R date object
  # date in format YYYYMM
  year <- substr(date, 1, 4) 
  month <- substr(date, 5, 6) 
  date <- paste(year, month, sep="-")
  date <- as.yearmon(date, format="%Y-%m")
  return(date)
}
